
//// KONTROLA RODNEHO CISLA

CREATE OR REPLACE TRIGGER rodne_cislo
  BEFORE INSERT OR UPDATE ON osoba
  FOR EACH ROW WHEN ((new.rodne_c IS NOT NULL) AND (MOD(new.rodne_c,11)<>0))
BEGIN
  RAISE_APPLICATION_ERROR(-20201,'Spatne rodne cislo');
END rodne_cislo;
/

// OK:    INSERT INTO osoba VALUES (8,'08-APR-80','Oto','Valek',8004080216,NULL);
// ERROR: UPDATE osoba SET rodne_c=8004080217;



//// KONTROLA KAPACITY POKOJE

CREATE OR REPLACE PACKAGE pokoje_pkg AS
  TYPE tabletype IS TABLE OF mistnost.cislo%type INDEX BY binary_integer;
  pokoje tabletype;
  pokoju binary_integer;
END;
/

CREATE OR REPLACE PROCEDURE pokoje_kontrola AS
  obsazeno integer;
  kapacita integer;
  pokoj integer;
BEGIN
  FOR i IN 1..(pokoje_pkg.pokoju) LOOP
    pokoj := pokoje_pkg.pokoje(i);
    SELECT kapacita INTO kapacita FROM pokoj WHERE mistnost_cislo = pokoj;
    SELECT count(ubytovan_osoba_cislo) INTO obsazeno FROM ubytovani WHERE obyvan_mistnost_cislo = pokoj;
    IF (obsazeno > kapacita) THEN
      RAISE_APPLICATION_ERROR(-20202,'Kapacita pokoje prekrocena');
    END IF;
  END LOOP;
END pokoje_kontrola;
/

CREATE OR REPLACE TRIGGER kapacita_pokoje1_a
  BEFORE INSERT OR UPDATE ON ubytovani
BEGIN
  pokoje_pkg.pokoju := 0;
END kapacita_pokoje1_a;
/

CREATE OR REPLACE TRIGGER kapacita_pokoje1_b
  BEFORE INSERT OR UPDATE ON ubytovani
  FOR EACH ROW
BEGIN
  pokoje_pkg.pokoju := pokoje_pkg.pokoju + 1;
  pokoje_pkg.pokoje(pokoje_pkg.pokoju) := :new.obyvan_mistnost_cislo;
END kapacita_pokoje1_b;
/

CREATE OR REPLACE TRIGGER kapacita_pokoje1_c
  AFTER INSERT OR UPDATE ON ubytovani
BEGIN
  pokoje_kontrola;
END kapacita_pokoje1_a;
/

CREATE OR REPLACE TRIGGER kapacita_pokoje2_a
  BEFORE INSERT OR UPDATE ON pokoj
BEGIN
  pokoje_pkg.pokoju := 0;
END kapacita_pokoje2_a;
/

CREATE OR REPLACE TRIGGER kapacita_pokoje2_b
  BEFORE INSERT OR UPDATE ON pokoj
  FOR EACH ROW
BEGIN
  pokoje_pkg.pokoju := pokoje_pkg.pokoju + 1;
  pokoje_pkg.pokoje(pokoje_pkg.pokoju) := :new.mistnost_cislo;
END kapacita_pokoje2_b;
/

CREATE OR REPLACE TRIGGER kapacita_pokoje2_c
  AFTER INSERT OR UPDATE ON pokoj
BEGIN
  pokoje_kontrola;
END kapacita_pokoje2_c;
/

// OK:    UPDATE ubytovani SET obyvan_mistnost_cislo=202 WHERE ubytovan_osoba_cislo=4;
// ERROR: UPDATE ubytovani SET obyvan_mistnost_cislo=302 WHERE ubytovan_osoba_cislo=6;
// OK:    UPDATE pokoj SET kapacita=10 WHERE mistnost_cislo=201;
// ERROR: UPDATE pokoj SET kapacita=2 WHERE mistnost_cislo=302;



//// KONTROLA KAPACITY LECEBNY

CREATE OR REPLACE PACKAGE lecebny_pkg AS
  TYPE dobytype IS TABLE OF procedura.doba%type INDEX BY binary_integer;
  TYPE lecebnatype IS TABLE OF lecebna.mistnost_cislo%type INDEX BY binary_integer;
  doby dobytype;
  lecebny lecebnatype;
  leceben binary_integer;
END;
/

CREATE OR REPLACE PROCEDURE lecebny_kontrola AS
  obsazeno integer;
  kapacita integer;
  v_lecebna integer;
  v_doba procedura.doba%type;
BEGIN
  FOR i IN 1..(lecebny_pkg.leceben) LOOP
    v_lecebna := lecebny_pkg.lecebny(i);
    v_doba := lecebny_pkg.doby(i);
    SELECT kapacita INTO kapacita FROM lecebna WHERE mistnost_cislo = v_lecebna;
    BEGIN
      SELECT count(konana_cislo) INTO obsazeno FROM konani, procedura
        WHERE cislo=konana_cislo AND doba=v_doba AND realizuje_mistnost_cislo=v_lecebna;
      IF (obsazeno > kapacita) THEN
        RAISE_APPLICATION_ERROR(-20203,'Kapacita lecebny prekrocena');
      END IF;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN NULL;
    END;
  END LOOP;
END lecebny_kontrola;
/

CREATE OR REPLACE TRIGGER kapacita_lecebny_a
  BEFORE INSERT OR UPDATE ON konani
BEGIN
  lecebny_pkg.leceben := 0;
END kapacita_lecebny_a;
/

CREATE OR REPLACE TRIGGER kapacita_lecebny_b
  BEFORE INSERT OR UPDATE ON konani
  FOR EACH ROW
DECLARE
  doba procedura.doba%type;
BEGIN
  SELECT doba INTO doba FROM procedura WHERE cislo = :new.konana_cislo;
  lecebny_pkg.leceben := lecebny_pkg.leceben + 1;
  lecebny_pkg.lecebny(lecebny_pkg.leceben) := :new.realizuje_mistnost_cislo;
  lecebny_pkg.doby(lecebny_pkg.leceben) := doba;
END kapacita_lecebny_b;
/

CREATE OR REPLACE TRIGGER kapacita_lecebny_c
  AFTER INSERT OR UPDATE ON konani
BEGIN
  lecebny_kontrola;
END kapacita_lecebny_c;
/

// OK:    UPDATE procedura SET doba='29-MAY-03' WHERE cislo=12;
//        UPDATE procedura SET doba='29-MAY-03' WHERE cislo=1;
//        UPDATE konani SET realizuje_mistnost_cislo=103 WHERE konana_cislo=12;
// ERROR: UPDATE konani SET realizuje_mistnost_cislo=103 WHERE konana_cislo=1;



//// KONTROLA KVALIFIKACE SESTRY PRO PROCEDURU

CREATE OR REPLACE TRIGGER kvalifikace
  BEFORE INSERT OR UPDATE ON provedeni_p
  FOR EACH ROW
DECLARE
  druh integer;
  dummy integer;
BEGIN
  SELECT druh_cislo INTO druh FROM procedura WHERE cislo=:new.provedena__cislo;
  SELECT NULL INTO dummy FROM kvalifikace WHERE ovlada_osoba_cislo=:new.provadi_p_osoba_cislo AND ovladana_cislo=druh;
EXCEPTION
  WHEN NO_DATA_FOUND THEN RAISE_APPLICATION_ERROR(-20204,'Sestra nema pro tuto proceduru kvalifikaci');
END kvalifikace;
/

// OK:    INSERT INTO provedeni_p VALUES (13,5);
// ERROR: INSERT INTO provedeni_p VALUES (12,7);



//// KONTROLA VYBAVENI LECEBNY PRO PROCEDURU

CREATE OR REPLACE TRIGGER vybaveni
  BEFORE INSERT OR UPDATE ON konani
  FOR EACH ROW
DECLARE
  druh integer;
  dummy integer;
BEGIN
  SELECT druh_cislo INTO druh FROM procedura WHERE cislo=:new.konana_cislo;
  SELECT NULL INTO dummy FROM vybaveni WHERE vybavena_mistnost_cislo=:new.realizuje_mistnost_cislo AND proveditel_cislo=druh;
EXCEPTION
  WHEN NO_DATA_FOUND THEN RAISE_APPLICATION_ERROR(-20205,'Lecebna nema pro proceduru vybaveni');
END vybaveni;
/

// OK:    INSERT INTO konani VALUES (5,101);
// ERROR: INSERT INTO konani VALUES (8,102);



//// KONTROLA STEJNEHO POHLAVI HOSTU NA POKOJI

CREATE OR REPLACE PACKAGE pohlavi_pkg AS
  TYPE tabletype IS TABLE OF mistnost.cislo%type INDEX BY binary_integer;
  pokoje tabletype;
  pokoju binary_integer;
END;
/

CREATE OR REPLACE PROCEDURE pohlavi_kontrola AS
  pohlavi integer;
  pokoj integer;
BEGIN
  FOR i IN 1..(pohlavi_pkg.pokoju) LOOP
    pokoj := pohlavi_pkg.pokoje(i);
    DECLARE
    CURSOR cur IS SELECT pohlavi FROM ubytovani, host
      WHERE ubytovan_osoba_cislo=osoba_cislo AND obyvan_mistnost_cislo=pokoj;
    BEGIN
      pohlavi := -1;
      FOR rec IN cur LOOP
        IF pohlavi<>-1 AND rec.pohlavi<>pohlavi THEN
          RAISE_APPLICATION_ERROR(-20206,'Na pokoji jsou hoste ruzneho pohlavi');
	ELSE
	  pohlavi := rec.pohlavi;
	END IF;
      END LOOP;
    END;
  END LOOP;
END pohlavi_kontrola;
/

CREATE OR REPLACE TRIGGER pohlavi1_a
  BEFORE INSERT OR UPDATE ON ubytovani
BEGIN
  pohlavi_pkg.pokoju := 0;
END pohlavi1_a;
/

CREATE OR REPLACE TRIGGER pohlavi1_b
  BEFORE INSERT OR UPDATE ON ubytovani
  FOR EACH ROW
BEGIN
  pohlavi_pkg.pokoju := pohlavi_pkg.pokoju + 1;
  pohlavi_pkg.pokoje(pohlavi_pkg.pokoju) := :new.obyvan_mistnost_cislo;
END pohlavi1_b;
/

CREATE OR REPLACE TRIGGER pohlavi1_c
  AFTER INSERT OR UPDATE ON ubytovani
BEGIN
  pohlavi_kontrola;
END pohlavi1_c;
/

CREATE OR REPLACE TRIGGER pohlavi2_a
  BEFORE INSERT OR UPDATE ON host
BEGIN
  pohlavi_pkg.pokoju := 0;
END pohlavi1_a;
/

CREATE OR REPLACE TRIGGER pohlavi2_b
  BEFORE INSERT OR UPDATE ON host
  FOR EACH ROW
DECLARE 
  pokoj integer;
BEGIN
  SELECT obyvan_mistnost_cislo INTO pokoj FROM ubytovani WHERE ubytovan_osoba_cislo=:new.osoba_cislo;
  IF :old.pohlavi<>:new.pohlavi THEN
    pohlavi_pkg.pokoju := pohlavi_pkg.pokoju + 1;
    pohlavi_pkg.pokoje(pohlavi_pkg.pokoju) := pokoj;
  END IF;
END pohlavi2_b;
/

CREATE OR REPLACE TRIGGER pohlavi2_c
  AFTER INSERT OR UPDATE ON host
BEGIN
  pohlavi_kontrola;
END pohlavi2_c;
/

// OK:    UPDATE ubytovani SET obyvan_mistnost_cislo=301 WHERE ubytovan_osoba_cislo=3;
// ERROR: UPDATE ubytovani SET obyvan_mistnost_cislo=301 WHERE ubytovan_osoba_cislo=1;
// OK:    UPDATE host SET pohlavi=1 WHERE osoba_cislo=6;
// OK:    UPDATE host SET pohlavi=0 WHERE osoba_cislo=4;
// ERROR: UPDATE host SET pohlavi=1 WHERE osoba_cislo=1;
